package it.polimi.dei.esercitazioniIngegneriaDelSoftware.swing.calculator;

/**
 * Un modello di calcolatrice semplificata. 
 */
public class CalculatorModel {

  /* ultimo risultato */  
  private double lastResult;

  /* ultimo operando ed operatore selezionati */
  private double currentOperand;
  private Operation currentOperation;

  /* stato della calcolatrice */
  private boolean hasOperand;
  private boolean hasOperation;

  /* User Interface */
  private CalculatorUI userInterface;

  /**
   * Crea un nuovo modello di calcolatrice
   */
  public CalculatorModel() {
    this.userInterface = null;
    this.reset();
  }

  /**
   * Imposta una User Interface 
   * @param userInterface una interfaccia utente per la calcolatrice.
   */
  public void setUI(CalculatorUI userInterface) {
    /* salvo riferimento alla UI */
    this.userInterface = userInterface;
    /* mi porto allo stato iniziale */
    this.reset();
  }

  /**
   * Imposta la calcolatrice nello stato iniziale.
   */
  public void reset() {
    this.lastResult = 0.0;

    /* imposta lo stato iniziale */
    this.hasOperand = false;
    this.hasOperation = false;

    /* imposta valori iniziali */
    this.currentOperand = 0.0;
    this.currentOperation = null;
    
    resetUI();
  }

  /**
   * Imposta l'operazione corrente
   * @param op operazione corrente.
   */
  public void setCurrentOperation(Operation op) {
    if (this.hasOperand)
      compute();

    this.hasOperand = false;
    this.hasOperation = true;

    this.currentOperation = op;

    this.updateUIafterOperation();
  }

  /**
   * Imposta l'operando corrente
   * @param value l'operando corrente
   */
  public void setOperand(double value) {
    this.hasOperand = true;
    this.currentOperand = value;
  }
  
  
  /**
   * Aggiorna il risultato usando operazione e operando corrente.
   * 
   * Questo metodo deve essere chiamato solo se l'operazione e' stata selezionata prima.
   * Se non e' stato selezionato un operando, viene utilizzato di default il risultato
   * della precedente operazione. 
   * 
   * @throws IllegalStateException se non e' stato selezionata una operazione.
   */
  public void compute() {
    if (!this.hasOperation)
      throw new IllegalStateException();

    double argument = 0.0;
    if (this.hasOperand)
      argument = this.currentOperand;
    else 
      argument = this.lastResult;

    this.lastResult = 
        this.currentOperation.perform(this.lastResult, argument);

    this.hasOperand = false;
    this.hasOperation = false;

    this.updateUIafterCompute();
  }
  
  // Aggiorna l'UI dopo un calcolo di risultato
  private void updateUIafterCompute() {
    if (this.userInterface != null) {
      this.userInterface.setValue(this.lastResult);
      this.userInterface.setDigits(false);
      this.userInterface.setEquals(false);
    }
  }
  
  // UI allo stato iniziale
  private void resetUI() {
    if (this.userInterface != null) {
      this.userInterface.setValue(0.0);
      this.userInterface.setDigits(false);
      this.userInterface.setEquals(false);
    }
  }
  
  // Aggiorna la UI dopo una operazione
  private void updateUIafterOperation() {
    if (this.userInterface != null) {
      this.userInterface.setValue(this.lastResult);
      this.userInterface.setDigits(true);
      this.userInterface.setEquals(true);
    }
  }
}
